@@ -194,8 +194,6 @@ parse_cmdline(int argc, char **argv, struct options *options,
 				if(g_file_test(system_path,
 						G_FILE_TEST_IS_REGULAR)) {
 					ret = config_read_file(system_path,error_r);
-					g_free(system_path);
-					g_free(&system_config_dirs);
 					break;
 				}
 				++i;;
