@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,34 +20,72 @@
 #ifndef DAEMON_H
 #define DAEMON_H
 
+#include "mpd_error.h"
+
 #include <stdbool.h>
 
+#ifndef WIN32
 void
-daemonize_init(const char *user, const char *pidfile);
+daemonize_init(const char *user, const char *group, const char *pidfile);
+#else
+static inline void
+daemonize_init(const char *user, const char *group, const char *pidfile)
+{ (void)user; (void)group; (void)pidfile; }
+#endif
 
+#ifndef WIN32
 void
 daemonize_finish(void);
+#else
+static inline void
+daemonize_finish(void)
+{ /* nop */ }
+#endif
 
 /**
  * Kill the MPD which is currently running, pid determined from the
  * pid file.
  */
+#ifndef WIN32
 void
 daemonize_kill(void);
+#else
+#include <glib.h>
+static inline void
+daemonize_kill(void)
+{ MPD_ERROR("--kill is not available on WIN32"); }
+#endif
 
 /**
  * Close stdin (fd 0) and re-open it as /dev/null.
  */
+#ifndef WIN32
 void
 daemonize_close_stdin(void);
+#else
+static inline void
+daemonize_close_stdin(void) {}
+#endif
 
 /**
  * Change to the configured Unix user.
  */
+#ifndef WIN32
 void
 daemonize_set_user(void);
+#else
+static inline void
+daemonize_set_user(void)
+{ /* nop */ }
+#endif
 
+#ifndef WIN32
 void
 daemonize(bool detach);
+#else
+static inline void
+daemonize(bool detach)
+{ (void)detach; }
+#endif
 
 #endif
