@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -50,6 +50,9 @@ enum {
 
 	/** a sticker has been modified. */
 	IDLE_STICKER = 0x80,
+
+	/** a database update has started or finished. */
+	IDLE_UPDATE = 0x100,
 };
 
 /**
