@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,23 +27,11 @@
 #ifndef MPD_INPUT_REWIND_H
 #define MPD_INPUT_REWIND_H
 
-#include "config.h"
+#include "check.h"
 
 struct input_stream;
 
-#ifdef HAVE_CURL
-
-void
+struct input_stream *
 input_rewind_open(struct input_stream *is);
 
-#else
-
-static inline void
-input_rewind_open(struct input_stream *is)
-{
-	(void)is;
-}
-
-#endif
-
 #endif
