@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 #ifndef MPD_ZEROCONF_H
 #define MPD_ZEROCONF_H
 
-#include "config.h"
+#include "check.h"
 
 #ifdef HAVE_ZEROCONF
 
