@@ -20,6 +20,7 @@
 #include "config.h"
 #include "decoder_list.h"
 #include "decoder_api.h"
+#include "tag_pool.h"
 #include "input_init.h"
 #include "input_stream.h"
 #include "audio_format.h"
@@ -180,6 +181,8 @@ int main(int argc, char **argv)
 
 	g_log_set_default_handler(my_log_func, NULL);
 
+	tag_pool_init();
+
 	if (!input_stream_global_init(&error)) {
 		g_warning("%s", error->message);
 		g_error_free(error);
@@ -228,5 +231,7 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
+	tag_pool_deinit();
+
 	return 0;
 }
